@@ -23,14 +23,8 @@
  ***************************************************************************/
 #include "tool_setup.h"
 
-#include "strcase.h"
-
-#include "curlx.h"
-
 #include "tool_libinfo.h"
 
-#include "memdebug.h" /* keep this as LAST include */
-
 /* global variable definitions, for libcurl runtime info */
 
 static const char *no_protos = NULL;
@@ -127,7 +121,7 @@ static struct feature_name_presentp {
   {NULL,             NULL,                0}
 };
 
-static const char *fnames[sizeof(maybe_feature) / sizeof(maybe_feature[0])];
+static const char *fnames[CURL_ARRAYSIZE(maybe_feature)];
 const char * const *feature_names = fnames;
 size_t feature_count;
 
