@@ -23,9 +23,7 @@
  ***************************************************************************/
 #include "first.h"
 
-#include "memdebug.h"
-
-static CURLcode test_lib544(char *URL)
+static CURLcode test_lib544(const char *URL)
 {
   CURL *curl;
   CURLcode res = CURLE_OK;
@@ -56,7 +54,7 @@ static CURLcode test_lib544(char *URL)
   test_setopt(curl, CURLOPT_URL, URL);
 
   if(testnum == 545)
-    test_setopt(curl, CURLOPT_POSTFIELDSIZE, (long) sizeof(teststring));
+    test_setopt(curl, CURLOPT_POSTFIELDSIZE, (long)sizeof(teststring));
 
   test_setopt(curl, CURLOPT_COPYPOSTFIELDS, teststring);
 
@@ -67,11 +65,11 @@ static CURLcode test_lib544(char *URL)
   strcpy(teststring, "FAIL");
 
   {
-    CURL *handle2;
-    handle2 = curl_easy_duphandle(curl);
+    CURL *curl2;
+    curl2 = curl_easy_duphandle(curl);
     curl_easy_cleanup(curl);
 
-    curl = handle2;
+    curl = curl2;
   }
 
   /* Now, this is a POST request with binary 0 embedded in POST data. */
