@@ -30,35 +30,35 @@
 
 static void check_spbset(const char *name, const unsigned int *s, size_t slen)
 {
-  struct uint_spbset bset;
+  struct uint32_spbset bset;
   size_t i, j;
   unsigned int n, c;
 
   curl_mfprintf(stderr, "test %s, %zu numbers\n", name, slen);
 
-  Curl_uint_spbset_init(&bset);
+  Curl_uint32_spbset_init(&bset);
 
-  Curl_uint_spbset_clear(&bset);
-  c = Curl_uint_spbset_count(&bset);
+  Curl_uint32_spbset_clear(&bset);
+  c = Curl_uint32_spbset_count(&bset);
   fail_unless(c == 0, "set count is not 0");
 
   for(i = 0; i < slen; ++i) { /* add all */
-    fail_unless(Curl_uint_spbset_add(&bset, s[i]), "failed to add");
+    fail_unless(Curl_uint32_spbset_add(&bset, s[i]), "failed to add");
     for(j = i + 1; j < slen; ++j)
-      fail_unless(!Curl_uint_spbset_contains(&bset, s[j]),
+      fail_unless(!Curl_uint32_spbset_contains(&bset, s[j]),
                   "unexpectedly found");
   }
 
   for(i = 0; i < slen; ++i) { /* all present */
-    fail_unless(Curl_uint_spbset_contains(&bset, s[i]),
+    fail_unless(Curl_uint32_spbset_contains(&bset, s[i]),
                 "failed presence check");
   }
 
   /* iterator over all numbers */
-  fail_unless(Curl_uint_spbset_first(&bset, &n), "first failed");
+  fail_unless(Curl_uint32_spbset_first(&bset, &n), "first failed");
   fail_unless(n == s[0], "first not correct number");
   for(i = 1; i < slen; ++i) {
-    fail_unless(Curl_uint_spbset_next(&bset, n, &n), "next failed");
+    fail_unless(Curl_uint32_spbset_next(&bset, n, &n), "next failed");
     if(n != s[i]) {
       curl_mfprintf(stderr, "expected next to be %u, not %u\n", s[i], n);
       fail_unless(n == s[i], "next not correct number");
@@ -66,28 +66,30 @@ static void check_spbset(const char *name, const unsigned int *s, size_t slen)
   }
 
   for(i = 0; i < slen; i += 2) { /* remove every 2nd */
-    Curl_uint_spbset_remove(&bset, s[i]);
-    fail_unless(!Curl_uint_spbset_contains(&bset, s[i]), "unexpectedly found");
+    Curl_uint32_spbset_remove(&bset, s[i]);
+    fail_unless(!Curl_uint32_spbset_contains(&bset, s[i]),
+                "unexpectedly found");
   }
   for(i = 1; i < slen; i += 2) { /* others still there */
-    fail_unless(Curl_uint_spbset_contains(&bset, s[i]), "unexpectedly gone");
+    fail_unless(Curl_uint32_spbset_contains(&bset, s[i]), "unexpectedly gone");
   }
   /* The count is half */
-  c = Curl_uint_spbset_count(&bset);
-  fail_unless(c == slen/2, "set count is wrong");
+  c = Curl_uint32_spbset_count(&bset);
+  fail_unless(c == slen / 2, "set count is wrong");
 
-  Curl_uint_spbset_clear(&bset);
-  c = Curl_uint_spbset_count(&bset);
+  Curl_uint32_spbset_clear(&bset);
+  c = Curl_uint32_spbset_count(&bset);
   fail_unless(c == 0, "set count is not 0");
   for(i = 0; i < slen; i++) { /* none present any longer */
-    fail_unless(!Curl_uint_spbset_contains(&bset, s[i]), "unexpectedly there");
+    fail_unless(!Curl_uint32_spbset_contains(&bset, s[i]),
+                "unexpectedly there");
   }
 
   for(i = 0; i < slen; ++i) { /* add all again */
-    fail_unless(Curl_uint_spbset_add(&bset, s[i]), "failed to add");
+    fail_unless(Curl_uint32_spbset_add(&bset, s[i]), "failed to add");
   }
 
-  Curl_uint_spbset_destroy(&bset);
+  Curl_uint32_spbset_destroy(&bset);
 }
 
 static CURLcode test_unit3213(const char *arg)
